@@ -2,7 +2,7 @@ from datetime import datetime
 import dateutil.parser
 from copy import (copy, deepcopy)
 
-from pystac import STACError
+from pystac import (STACError, STACObjectType)
 from pystac.catalog import Catalog
 from pystac.link import Link
 from pystac.utils import datetime_to_str
@@ -24,15 +24,16 @@ class Collection(Catalog):
         href (str or None): Optional HREF for this collection, which be set as the collection's
             self link's HREF.
         license (str):  Collection's license(s) as a `SPDX License identifier
-            <https://spdx.org/licenses/>`_ or `expression
-            <https://spdx.org/spdx-specification-21-web-version#h.jxpfx0ykyb60>`_. Defaults
-            to 'proprietary'.
+            <https://spdx.org/licenses/>`_, `various`, or `proprietary`. If collection includes
+            data with multiple different licenses, use `various` and add a link for each.
+            Defaults to 'proprietary'.
         keywords (List[str]): Optional list of keywords describing the collection.
-        version (str): Optional version of the Collection.
         providers (List[Provider]): Optional list of providers of this Collection.
         properties (dict): Optional dict of common fields across referenced items.
         summaries (dict): An optional map of property summaries,
             either a set of values or statistics such as a range.
+        extra_fields (dict or None): Extra fields that are part of the top-level JSON properties
+            of the Collection.
 
     Attributes:
         id (str): Identifier for the collection.
@@ -42,14 +43,18 @@ class Collection(Catalog):
         title (str or None): Optional short descriptive one-line title for the collection.
         stac_extensions (List[str]): Optional list of extensions the Collection implements.
         keywords (List[str] or None): Optional list of keywords describing the collection.
-        version (str or None): Optional version of the Collection.
         providers (List[Provider] or None): Optional list of providers of this Collection.
         properties (dict or None): Optional dict of common fields across referenced items.
         summaries (dict or None): An optional map of property summaries,
             either a set of values or statistics such as a range.
         links (List[Link]): A list of :class:`~pystac.Link` objects representing
             all links associated with this Collection.
+        extra_fields (dict or None): Extra fields that are part of the top-level JSON properties
+            of the Catalog.
     """
+
+    STAC_OBJECT_TYPE = STACObjectType.COLLECTION
+
     DEFAULT_FILE_NAME = "collection.json"
     """Default file name that will be given to this STAC object in a cononical format."""
     def __init__(self,
@@ -59,19 +64,19 @@ class Collection(Catalog):
                  title=None,
                  stac_extensions=None,
                  href=None,
+                 extra_fields=None,
                  license='proprietary',
                  keywords=None,
-                 version=None,
                  providers=None,
                  properties=None,
                  summaries=None):
-        super(Collection, self).__init__(id, description, title, stac_extensions, href)
+        super(Collection, self).__init__(id, description, title, stac_extensions, extra_fields,
+                                         href)
         self.extent = extent
         self.license = license
 
         self.stac_extensions = stac_extensions
         self.keywords = keywords
-        self.version = version
         self.providers = providers
         self.properties = properties
         self.summaries = summaries
@@ -115,8 +120,6 @@ class Collection(Catalog):
             d['stac_extensions'] = self.stac_extensions
         if self.keywords is not None:
             d['keywords'] = self.keywords
-        if self.version is not None:
-            d['version'] = self.version
         if self.providers is not None:
             d['providers'] = list(map(lambda x: x.to_dict(), self.providers))
         if self.properties is not None:
@@ -131,65 +134,68 @@ class Collection(Catalog):
                            description=self.description,
                            extent=self.extent.clone(),
                            title=self.title,
-                           license=self.license,
                            stac_extensions=self.stac_extensions,
+                           extra_fields=self.extra_fields,
+                           license=self.license,
                            keywords=self.keywords,
-                           version=self.version,
                            providers=self.providers,
                            properties=self.properties,
                            summaries=self.summaries)
 
         clone._resolved_objects.cache(clone)
 
-        for l in self.links:
-            if l.rel == 'root':
+        for link in self.links:
+            if link.rel == 'root':
                 # Collection __init__ sets correct root to clone; don't reset
                 # if the root link points to self
-                root_is_self = l.is_resolved() and l.target is self
+                root_is_self = link.is_resolved() and link.target is self
                 if not root_is_self:
                     clone.set_root(None)
-                    clone.add_link(l.clone())
+                    clone.add_link(link.clone())
             else:
-                clone.add_link(l.clone())
+                clone.add_link(link.clone())
 
         return clone
 
     @classmethod
     def from_dict(cls, d, href=None, root=None):
-        id = d['id']
-        description = d['description']
-        license = d['license']
-        extent = Extent.from_dict(d['extent'])
+        d = deepcopy(d)
+        id = d.pop('id')
+        description = d.pop('description')
+        license = d.pop('license')
+        extent = Extent.from_dict(d.pop('extent'))
         title = d.get('title')
         stac_extensions = d.get('stac_extensions')
         keywords = d.get('keywords')
-        version = d.get('version')
         providers = d.get('providers')
         if providers is not None:
             providers = list(map(lambda x: Provider.from_dict(x), providers))
         properties = d.get('properties')
         summaries = d.get('summaries')
+        links = d.pop('links')
+
+        d.pop('stac_version')
 
         collection = Collection(id=id,
                                 description=description,
                                 extent=extent,
                                 title=title,
-                                license=license,
                                 stac_extensions=stac_extensions,
+                                extra_fields=d,
+                                license=license,
                                 keywords=keywords,
-                                version=version,
                                 providers=providers,
                                 properties=properties,
                                 summaries=summaries)
 
         has_self_link = False
-        for l in d['links']:
-            has_self_link |= l['rel'] == 'self'
-            if l['rel'] == 'root':
+        for link in links:
+            has_self_link |= link['rel'] == 'self'
+            if link['rel'] == 'root':
                 # Remove the link that's generated in Catalog's constructor.
                 collection.remove_links('root')
 
-            collection.add_link(Link.from_dict(l))
+            collection.add_link(Link.from_dict(link))
 
         if not has_self_link and href is not None:
             collection.add_link(Link.self_href(href))
@@ -310,8 +316,8 @@ class SpatialExtent:
             SpatialExtent: A SpatialExtent with a single bbox that covers the
             given coordinates.
         """
-        def process_coords(l, xmin=None, ymin=None, xmax=None, ymax=None):
-            for coord in l:
+        def process_coords(link, xmin=None, ymin=None, xmax=None, ymax=None):
+            for coord in link:
                 if type(coord[0]) is list:
                     xmin, ymin, xmax, ymax = process_coords(coord, xmin, ymin, xmax, ymax)
                 else:
