@@ -1,9 +1,88 @@
 from abc import (ABC, abstractmethod)
 
+import pystac
 from pystac import STACError
 from pystac.link import (Link, LinkType)
 from pystac.stac_io import STAC_IO
 from pystac.utils import (is_absolute_href, make_absolute_href)
+from pystac.extensions import ExtensionError
+
+
+class STACObjectType:
+    CATALOG = 'CATALOG'
+    COLLECTION = 'COLLECTION'
+    ITEM = 'ITEM'
+    ITEMCOLLECTION = 'ITEMCOLLECTION'
+
+
+class ExtensionIndex:
+    """Defines methods for accessing extension functionality.
+
+    To access a specific extension, use the __getitem__ on this class with the
+    extension ID::
+
+        # Access the "bands" property on the eo extension.
+        item.ext['eo'].bands
+    """
+    def __init__(self, stac_object):
+        self.stac_object = stac_object
+
+    def __getitem__(self, extension_id):
+        """Gets the extension object for the given extension.
+
+        Returns:
+            CatalogExtension or CollectionExtension or ItemExtension: The extension object
+            through which you can access the extension functionality for the extension represented
+            by the extension_id.
+        """
+        # Check to make sure this is a registered extension.
+        if not pystac.STAC_EXTENSIONS.is_registered_extension(extension_id):
+            raise ExtensionError("'{}' is not an extension "
+                                 "registered with PySTAC".format(extension_id))
+
+        if not self.implements(extension_id):
+            raise ExtensionError("{} does not implement the {} extension. "
+                                 "Use the 'ext.enable' method to enable this extension "
+                                 "first.".format(self.stac_object, extension_id))
+
+        return pystac.STAC_EXTENSIONS.extend_object(extension_id, self.stac_object)
+
+    def __getattr__(self, extension_id):
+        """Gets an extension based on a dynamic attribute.
+
+        This takes the attribute name and passes it to __getitem__.
+
+        This allows the following two lines to be equivalent::
+
+            item.ext["label"].label_properties
+            item.ext.label.label_properties
+        """
+        return self[extension_id]
+
+    def enable(self, extension_id):
+        """Enables a stac extension for the given object. If the object already
+        enables the extension, no action is taken. If it does not, the extension ID is
+        added to the object's stac_extension property.
+
+        Args:
+            extension_id (str): The extension ID representing the extension
+            the object should implement
+
+        """
+        pystac.STAC_EXTENSIONS.enable_extension(extension_id, self.stac_object)
+
+    def implements(self, extension_id):
+        """Returns true if the associated object implements the given extension.
+
+        Args:
+            extension_id (str): The extension ID to check
+
+        Returns:
+            [bool]: True if the object implements the extensions - i.e. if
+                the extension ID is in the "stac_extensions" property.
+        """
+        return (self.stac_object.stac_extensions is not None
+                and extension_id in self.stac_object.stac_extensions)
 
 
 class LinkMixin:
@@ -40,7 +119,7 @@ class LinkMixin:
              rel (str): The :class:`~pystac.Link` ``rel`` to match on.
         """
 
-        self.links = [l for l in self.links if l.rel != rel]
+        self.links = [link for link in self.links if link.rel != rel]
         return self
 
     def get_single_link(self, rel):
@@ -50,7 +129,7 @@ class LinkMixin:
              rel (str): The :class:`~pystac.Link` ``rel`` to match on.
         """
 
-        return next((l for l in self.links if l.rel == rel), None)
+        return next((link for link in self.links if link.rel == rel), None)
 
     def get_links(self, rel=None):
         """Gets the :class:`~pystac.Link` instances associated with this object.
@@ -66,7 +145,7 @@ class LinkMixin:
         if rel is None:
             return self.links
         else:
-            return [l for l in self.links if l.rel == rel]
+            return [link for link in self.links if link.rel == rel]
 
     def clear_links(self, rel=None):
         """Clears all :class:`~pystac.Link` instances associated with this object.
@@ -75,7 +154,7 @@ class LinkMixin:
             rel (str or None): If set, only clear links that match this relationship.
         """
         if rel is not None:
-            self.links = [l for l in self.links if l.rel != rel]
+            self.links = [link for link in self.links if link.rel != rel]
         else:
             self.links = []
         return self
@@ -85,18 +164,18 @@ class LinkMixin:
         This does not include the self link, as those must always be absolute.
         See :func:`Link.make_relative <pystac.Link.make_relative>` for more information.
         """
-        for l in self.links:
-            if l.rel != 'self':
-                l.make_relative()
+        for link in self.links:
+            if link.rel != 'self':
+                link.make_relative()
         return self
 
     def make_links_absolute(self):
         """Sets each link associated with this object to be absolute.
         See :func:`Link.make_absolute <pystac.Link.make_absolute>` for more information.
         """
-        for l in self.links:
-            if l.rel != 'self':
-                l.make_absolute()
+        for link in self.links:
+            if link.rel != 'self':
+                link.make_absolute()
         return self
 
     def get_self_href(self):
@@ -144,8 +223,20 @@ class STACObject(LinkMixin, ABC):
         links (List[Link]): A list of :class:`~pystac.Link` objects representing
             all links associated with this STACObject.
     """
-    def __init__(self):
+
+    STAC_OBJECT_TYPE = None  # Overridden by the child classes with their type.
+
+    def __init__(self, stac_extensions):
         self.links = []
+        self.stac_extensions = stac_extensions
+
+    def validate(self):
+        """Validate this STACObject.
+
+        Raises:
+            STACValidationError
+        """
+        return pystac.validation.validate(self)
 
     def get_root(self):
         """Get the :class:`~pystac.Catalog` or :class:`~pystac.Collection` to
@@ -260,12 +351,14 @@ class STACObject(LinkMixin, ABC):
                 link.resolve_stac_object(root=self.get_root())
                 yield link.target
 
-    def save_object(self, include_self_link=True):
+    def save_object(self, include_self_link=True, dest_href=None):
         """Saves this STAC Object to it's 'self' HREF.
 
         Args:
           include_self_link (bool): If this is true, include the 'self' link with this object.
               Otherwise, leave out the self link.
+          dest_href (str): Optional HREF to save the file to. If None, the object will be saved
+              to the object's self href.
 
         Raises:
             :class:`~pystac.STACError`: If no self href is set, this error will be raised.
@@ -275,11 +368,14 @@ class STACObject(LinkMixin, ABC):
             STAC best practices document
             <https://github.com/radiantearth/stac-spec/blob/v0.8.1/best-practices.md#use-of-links>`_
         """
-        self_href = self.get_self_href()
-        if self_href is None:
-            raise STACError('Self HREF must be set before saving.')
+        if dest_href is None:
+            self_href = self.get_self_href()
+            if self_href is None:
+                raise STACError(
+                    'Self HREF must be set before saving without an explicit dest_href.')
+            dest_href = self_href
 
-        STAC_IO.save_json(self.get_self_href(), self.to_dict(include_self_link=include_self_link))
+        STAC_IO.save_json(dest_href, self.to_dict(include_self_link=include_self_link))
 
     def full_copy(self, root=None, parent=None):
         """Create a full copy of this STAC object and any stac objects linked to by
@@ -321,6 +417,23 @@ class STACObject(LinkMixin, ABC):
 
         return clone
 
+    @property
+    def ext(self):
+        """Access extensions for this STACObject.
+
+        Example:
+            This example shows accessing a Item's EO extension functionality
+            that gets the band information for an asset::
+
+                item = pystac.read_file("eo_item.json")
+                bands = item.ext.eo.get_asset_bands(item.assets["image"])
+
+        Returns:
+            ExtensionIndex: The object that can be used to access extension information
+            and functionality.
+        """
+        return ExtensionIndex(self)
+
     @abstractmethod
     def fully_resolve(self):
         """Ensure all STACObjects linked to by this STACObject are
